/* filename:    vb_irf.g
** description: The program computes impulse response 
**              functions for the VAR model
** created:     11/06/01
*/

new;
closeall;
cls;
outwidth 256;

format /mat /on /mb1 /ros 16,8;

/** Global defaults **/
  __output = 1;
  _fcmptol = 1E-10;
  _verbose = 1;           /** 1 for verbose **/

#include vb_spec.g; 
#include pathspec.g;

/********************************************************************
* Load Parameter Draws from Posterior Sampler Output
********************************************************************/
mhrun  = "1";      /* "0" for draws from prior; "1" for draws from posterior */
nblock = 1000;     /* block size */
nuse   = 1;       /* use every nuse observation */

lpath  = parapath $+ "\\mhrun" $+ mhrun;
opath  = resupath $+ "\\mhrun" $+ mhrun;
chkdir = chk_dir(opath);

lpara  = lpath $+ "\\paravar";

open fhpara = ^lpara for read;

/* Initialize Output files
*/
opath     = resupath $+ "\\mhrun" $+ mhrun;
chkdir    = chk_dir(opath);

ovarirf   = opath $+ "\\varirf";

create fhvarirf = ^ovarirf with IRF, nvar_*nirf_, 8;

/* Initialization
*/ 
eofloop = 0;
loopct  = 1;

k = 1+nlags_*nvar_;

"Generating IRFs...";

do until eofloop; 

   /** Read blocks of size nblock **/
   parasim  = readr(fhpara,nblock);
   nread    = rows(parasim);
   npara    = cols(parasim);

   /** use every nuse-th draw **/
   indseq  = seqa(1,1,nread);
   indseq  = indseq % nuse;
   indseq  = indseq .ne 0;

   parasim = delif(parasim,indseq);
   nread   = rows(parasim);

   if ismiss(parasim) == 0;

   " ";
   "Loop      " loopct;
   "Blocksize " nread;
   eofloop = eof(fhpara);

   irfsim = zeros(1,nvar_*nirf_);
   
   j = 1;
   do until j > rows(parasim);
   
      if fmod(j,nread/10)==0;".";; pause(1e-20); endif;
   
      phi      = reshape(parasim[j,1:nvar_*k],nvar_,k)';
      phi      = phi[1:nvar_*nlags_,.]; /* remove intercept if present */
      sigmatrq = reshape(parasim[j,nvar_*k+1:npara],nvar_,1)';
      {valid, YYirf} = varirf(phi,sigmatrq,signrestrictions_,nirf_);

      if valid == 1;
         irfsim = irfsim | vec(YYirf)';
      endif;
         
      j = j+1;
  endo;
      
  /* Write Output
  */
  if rows(irfsim) > 1;
     wr = writer(fhvarirf,irfsim[2:rows(irfsim),.]);
  endif;
  "";
  "Valid responses" rows(irfsim)-1;

  loopct = loopct + 1;
  endif;

endo;

closeall;

"";
"DONE";
end;

/********************************************************************
**   Procedure to compute Impulse Response Function
********************************************************************/
proc (2) = varirf(Phi,Sigmatrq,signrestrictions,nirf);
local nvar, nlags, imp, YYirf, XXpred, j, YYirfcheck, valid;

/* Initializations
*/
nvar   = cols(Phi);
nlags  = rows(Phi)/nvar;
YYirf  = zeros(nirf,nvar);
XXpred = zeros(1,nvar*nlags);
imp    = Sigmatrq;

/* Compute Impulse Responses
*/
j = 1;
do until j > nirf;

   if j == 1;
      YYirf[j,.] = XXpred*phi + imp;
   else;
      YYirf[j,.] = XXpred*phi;
   endif;
   
   if nlags > 1;
      XXpred[nvar+1:nvar*nlags] = XXpred[1:(nlags-1)*nvar];
   endif;
   XXpred[1:nvar] = YYirf[j,.];
   
   j = j+1;
   
endo;

/* Check whether sign restrictions are satisfied
** coding of signrestrictions
** 0 = no restriction, 1 = response >= 0, -1 = response <=0
** violation of sign restriction:  irf*restriction < 0
*/
YYirfcheck = YYirf[1:rows(signrestrictions),.] .* signrestrictions;
YYirfcheck = YYirfcheck .< 0;
valid = 1-maxc(maxc(YYirfcheck));

retp(valid, YYirf);
endp;

