/******************************************************************/
/* filename :       vf_irfcs_qgrid.g                              */
/* Description:                                                   */
/*                                                                */
/******************************************************************/

new;
closeall; 
cls;

#include vf_spec.g; 
#include pathspec.g;

// Define some constants 
critval_nsim_  = 5000;
lambda_nrep_   = 2000;
freqrun        = "2"; 

// Load the confidence interval for q
nblock = 100;
lpath  = resupath $+ "\\freqrun" $+ freqrun;
lqcs   = lpath $+ "\\" $+ "qcs";
open fhqcs = ^lqcs for read;
q_cs   = readr(fhqcs,1);
eofloop = 0;

do until eofloop;

   q_cs = q_cs | readr(fhqcs,nblock);
   eofloop = eof(fhqcs);

endo;
closeall;

nq_ = rows(q_cs);

// Specify IRFs for which CI's are computed
// variables horizons
irfindex_ = ( seqa(1,1,nvar_) .*. ones(nirf_,1) ) ~ ( ones(nvar_,1).*.seqa(1,1,nirf_) );

// Import data
#include vf_loaddata.g;
k = 1+nvar_*nlags_;

// Estimate normalized impulse responses
"Computing Impulse Responses";
pause(1e-5);
{ YYirf, phi_hat, sigma_hat } = estimate_irfs(YY,XX,nirf_);
  YYirfselect                 = select_irfs(YYirf,irfindex_);
psi_hat_   = vec(YYirfselect');

// estimate lambda (covariance matrix of phi_hat)
// we are using a fixed seed for the bootstrap of phi_hat
"Estimating Lambda via Bootstrap";
lambda_hat_ = estimate_lambda(YY,XX,lambda_nrep_,irfindex_);

// initialization of output files
varirf_freq_l = zeros(nirf_,nvar_);
varirf_freq_h = zeros(nirf_,nvar_);

/******************************************************** 
* Iterate over variables/horizons                       *
********************************************************/

/** start log file **/
outstr = resupath $+ "\\" $+ ftos(timedt,"%*.*lf",1,0) $+ "freqirf" $+ freqrun $+ ".log"; 
output file = ^outstr reset;

varindx = 1;
do until varindx > nvar_;

   //horizonindx = 8;
   horizonindx = 1;
   do until horizonindx > nirf_;
   
      varirf_temp_l = zeros(nq_,1);
      varirf_temp_h = zeros(nq_,1);

      /** set start time **/
      tstart = date;   

      q_ind = 1;
      do until q_ind > nq_;
         // CI conditional on q

         q = q_cs[q_ind,.]';

         // fix this
         psi_hat_ind = ((varindx-1)*nirf_+horizonindx-1 )*nvar_;
         th_hat      = psi_hat_[psi_hat_ind+1:psi_hat_ind+nvar_,1]'*q;
         th_se       = sqrt( q'*lambda_hat_[psi_hat_ind+1:psi_hat_ind+nvar_,psi_hat_ind+1:psi_hat_ind+nvar_]*q);

         varirf_temp_l[q_ind] = th_hat - cdfni(1-(1-tauth_)/2)*th_se;
         varirf_temp_h[q_ind] = th_hat + cdfni(1-(1-tauth_)/2)*th_se; 

         //q_ind q' varirf_temp_h[q_ind] th_hat th_se;

         q_ind = q_ind+1;
      endo;

      // take union of CIs

      varirf_freq_l[horizonindx,varindx] = minc(varirf_temp_l);
      varirf_freq_h[horizonindx,varindx] = maxc(varirf_temp_h);

      "============================================";
      // truncate CI in view of sign restr
      if horizonindx le nirfsign_;
         if signrestrictions_[horizonindx,varindx] == -1;
            // response is restricted to be <= 0
            "Imposed Upper Bound on CI";
            varirf_freq_l[horizonindx,varindx] = minc( 0 | varirf_freq_l[horizonindx,varindx] );
            varirf_freq_h[horizonindx,varindx] = minc( 0 | varirf_freq_h[horizonindx,varindx] );    
         elseif signrestrictions_[horizonindx,varindx] == 1;
            "Imposed Lower Bound on CI";
            // response is restricted to be >= 0
            varirf_freq_l[horizonindx,varindx] = maxc( 0 | varirf_freq_l[horizonindx,varindx] );
            varirf_freq_h[horizonindx,varindx] = maxc( 0 | varirf_freq_h[horizonindx,varindx] );    
         endif;
      endif;

      /** set finish time **/
      tfin = date;
      "Elapsed Time"; etstr(ethsec(tstart,tfin));
      "Variable" varindx "Horizon" horizonindx;
      "Freq Intv:  "  varirf_freq_l[horizonindx,varindx]~varirf_freq_h[horizonindx,varindx];       
      "Binding q:  "  minindc(varirf_temp_l)~maxindc(varirf_temp_h);
      "============================================";
      pause(1e-2);

      //end;

      horizonindx = horizonindx + 1;

    endo;

    varindx = varindx + 1;

endo;

opath  = resupath $+ "\\freqrun" $+ freqrun;
odraws = opath $+ "\\" $+ "varirfs"; 
chk_dir(opath);

create fhdraws = ^odraws with IRF, nvar_*nirf_, 8;
writer(fhdraws,( vec(varirf_freq_l)'|vec(varirf_freq_h)' ));
closeall;

output off;

end;

/***************************************************************/

#include simulate_var.src
#include estimate_lambda.src
#include estimate_irfs.src
#include select_irfs.src






